package editor;
import gui.CanvasDialogue;

import java.awt.Canvas;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import utils.Global;

/**
 * Builds a non-resizable confirmation dialogue made up of text fields uniquely identified. 
 * Regular expressions are used to describe the valid format of each field's contents. When 
 * that content is wrong, the background colour of the field in question is changed and the 
 * ok button is disabled.
 * @author hqcasanova
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class FieldConfirm extends CanvasDialogue implements ActionListener, DocumentListener {
	private JButton cancelButton, okButton;
	private JTextField[] fields;		//text fields
	private String[] fieldsContents;	//contents of each field
	private String[] correctFormats;	//regular expressions describing correctness
	private boolean  bAccepted,			//ok button pressed
	             	 bFilled;      		//all fields filled in
	private int iNumberFields;		   	//total number of fields 
	
	/**
	 * Customised constructor. Sets up all the text fields, together with the confirmation buttons. 
	 * @param sTitle Confirmation dialogue's title
	 * @param fieldTitles Title of each text field. Can be null. 
	 * @param correctFormats Regular expressions describing correct field formats.
	 */
	public FieldConfirm (String sTitle, String[] fieldTitles, String[] correctFormats) {
		//Non-resizable empty window without background and actionless "x" button
		super (sTitle, null, null, null, null, false, JDialog.DISPOSE_ON_CLOSE, 
			   Global.FAVICON_ED, true);
	    int i;
	    
	    this.correctFormats = correctFormats;

	    //Confirmation buttons
		cancelButton = new JButton (Global.LANG.getString ("botonCancelar"));
		cancelButton.addActionListener (this);
		okButton = new JButton (Global.LANG.getString ("botonAceptar"));
		okButton.setEnabled (false);
		okButton.addActionListener (this);
				
		//Sets up fields
		if (fieldTitles == null)
		   iNumberFields = 0;
		else {
		     iNumberFields = fieldTitles.length;
		     fields = new JTextField[iNumberFields];
		     fieldsContents = new String[iNumberFields];
		     for (i = 0; i < iNumberFields; i++) {
			     fields[i] = new JTextField();
			     fields[i].getDocument().addDocumentListener (this);
			     fields[i].getDocument().putProperty ("id", i);
			     fields[i].setText("");
			     fieldsContents[i] = "";
		     }		     
		}
		
		//Assigns components to window areas
		Component[] dummy = new Component[1];
		dummy[0] = new JLabel ();
		Component[] bottomRight = new Component[2];
		bottomRight[0] = okButton;
		bottomRight[1] = cancelButton;
		
		//Add components to the window
		if (iNumberFields != 0) {
		   JPanel centro = new JPanel (new GridLayout (0, 1, 0, 10));
		   for (i = 0; i < iNumberFields; i++)
			   centro.add (CanvasDialogue.titledFrame (fieldTitles[i], fields[i], null, true));
		   populate (dummy, dummy, false, centro, dummy, bottomRight);
		   removeTopPanel();
		}		
	}
	
	/**
	 * Overloaded customised constructor. Case for a single text field.
	 * @param sTitle Confirmation dialogue's title
	 * @param fieldTitle Title of the text field. Can be null. 
	 * @param correctFormat Correct format of field's contents
	 */
	public FieldConfirm (String sTitle, String fieldTitle, String correctFormat) {
		//Non-resizable empty window without background and actionless "x" button
		super (sTitle, null, Global.DEFAULT_BORDER, null, null, false, 
			   JDialog.DISPOSE_ON_CLOSE, Global.FAVICON_ED, true, true, false);
	    	    
	    //Initialise correct format description
	    String[] tipoVector = new String[1];
	    tipoVector[0] = correctFormat;
	    this.correctFormats = tipoVector;

	    //Initialise buttons
		cancelButton = new JButton (Global.LANG.getString ("botonCancelar"));
		cancelButton.addActionListener (this);
		okButton = new JButton (Global.LANG.getString ("botonAceptar"));
		okButton.setEnabled (false);
		okButton.addActionListener (this);
		
		//Initialise field
		if (fieldTitle == null)
		   iNumberFields = 0;
		else {
		     iNumberFields = 1;
		     fields = new JTextField[iNumberFields];
		     fieldsContents = new String[iNumberFields];
		     fields[0] = new JTextField();
		     fields[0].getDocument().addDocumentListener (this);
		     fields[0].getDocument().putProperty ("id", 0);
		     fields[0].setText("");
		     fieldsContents[0] = "";		     		     
		}
		
		//Assigns components to window areas
		Component[] topLeft, topRight; 
		Component[] bottomRight = {okButton, cancelButton};
		Component[] dummy = {new JLabel ()};
		if (iNumberFields != 0) {
		   topLeft = new Component[1];
		   topLeft[0] = new JLabel (fieldTitle);
		   topRight = new Component[1];
		   topRight[0] = fields[0];	   		   
		} else topLeft = topRight = null;
				
		//Adds components to the window
		populate (topLeft, topRight, false, dummy[0], dummy, bottomRight);			
	}
	
	/**
	 * Retrieves the contents of all fields
	 * @return Vector de valores
	 */
	protected String[] getFields () {
		return fieldsContents;
	}
	
	/**
	 * Resets all flags and empties all text fields.
	 */
	protected void reset() {
		int i;
		
		for (i = 0; i < iNumberFields; i++) {
			fields[i].setText("");
		}
		bFilled = false;
		bAccepted = false;
		getWindow().getRootPane().setDefaultButton (okButton);
	}
	
	/**
	 * Checks the contents of a text field against the corresponding regular expression describing
	 * format validity. 
	 * @param iFieldId ID of the field to be validated
	 * @return True if the field is correct. False if no;
	 */
	protected boolean isCorrect (int iFieldId) {
		return fieldsContents[iFieldId].matches (correctFormats[iFieldId]);
	}
	
	/**
	 * Checks if all fields (except the one currently being filled in) have been filled in correctly.
	 * @param iChangedField Id of the field that has just changed. -1 for no exceptions
	 * @return True if all fields are correct. False if not.
	 */
	private boolean isAllFilled (int iChangedField) {
		int i;
				
		bFilled = true;
		if (iNumberFields != 0) {  //Any fields?
		   i = 0;
		   while ((i < iNumberFields) && bFilled) {  //As soon as it finds one field empty => false
			     if (i != iChangedField) 
			        bFilled = (!fieldsContents[i].trim().isEmpty()) && isCorrect(i) && bFilled;
			     i++;
	       }
		}
	    return bFilled;
	}
		
	/**
	 * Updates the array holding field values and checks they are all correctly filled in. If so,
	 * the ok button is enabled. If not, apart from disabling the button, the background colour of 
	 * the field detected as incorrect is changed.
	 * @param iFieldId ID of the field that has just changed
	 */
	private void checkFields (int iFieldId) {
		//Updates the array of field values
        fieldsContents[iFieldId] = fields[iFieldId].getText();
		
  	    //Correct format? => correct-version background and, if remaining fields correct too, enable ok 
		if (isCorrect (iFieldId)) {
		   fields[iFieldId].setBackground (UIManager.getColor ("TextField.background"));	  
		   if ((iNumberFields == 1) || ((iNumberFields > 1) && (isAllFilled (iFieldId)))) 
			  okButton.setEnabled (true);
		   				  
		//Incorrect format? => incorrect-version background and disable ok button
		} else if (!fieldsContents[iFieldId].trim().isEmpty()) {
		          fields[iFieldId].setBackground (Global.FIELD_ERROR_COLOR);
		          okButton.setEnabled (false);		          
		
		//Is the field empty? => disable ok button and switch to correct-version background
		} else {			 
			   okButton.setEnabled (false);
			   fields[iFieldId].setBackground (UIManager.getColor ("TextField.background"));	  
		}			
	}
	
	/**
	 * Resets the dialogue (field background colour and ok button) and displays it
	 * @param owner Dialogue's owner. When displayed, the dialogue will center itself relative to the owner.
	 * @return True if the ok button was pressed. False if not.
	 */	
	protected boolean muestraCampoConfirmar (Window owner) {
		reset();
		display (owner);
		return bAccepted;
	}
	
	/**
	 * Manages events from the confirmation buttons. The strategy is the same for both: 
	 * updates the confirmation flag and closes the window. 
	 */
	public void actionPerformed (ActionEvent e) {
		if (e.getSource() == cancelButton) {
		   bAccepted = false;
		   close();
				
		} else if (e.getSource() == okButton) {
			      bAccepted = true;
			      close();	
		}		       
	}

	//Check validity of all fields every time something is typed in 
	public void insertUpdate(DocumentEvent e) {
		checkFields ((Integer) e.getDocument().getProperty ("id"));		
	}

	public void removeUpdate(DocumentEvent e) {
		checkFields ((Integer) e.getDocument().getProperty ("id"));		
	}
	
	public void changedUpdate(DocumentEvent e) {}	
}
