package editor;

import gui.Canvas;
import gui.ImageArray;

import java.awt.Component;
import java.awt.Cursor;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import javax.swing.Timer;

import utils.Data;
import utils.Global;

/**
 * Builds a confirmation dialogue used to name new documents and choose a representative 
 * pictogram. To that end, a text field and a pictogram selection list are provided. 
 * The contents of the text field are validated on key press. Format errors are shown, 
 * the ok button being disabled in such cases.
 * @author hquintero
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class FieldPictoConfirm extends FieldConfirm {
	private JList pictoList;		//pictogram selection list
	private DefaultListModel model; //data model governing list contents	
	private Timer cronoText;		//timer for text field's response
    private int iIcoDim,			//dimensions for all icons representing items
				iNumItems;	    	//number of list items in a row      
	
	private String[] ficherosItems;	 //maps list index of pictogram to id (its filename)
	
	/**
	 * Customised constructor. Sets up the dialogue with the text field, the pictogram selection
	 * list and the confirmation buttons. It also configures the real-time validation system.
	 * @param sTitle Title of the dialogue
	 * @param sFieldTitle Heading for the text field
	 * @param sPictoListTitle Heading for the pictogram selection list
	 * @param sCorrectFormat Regular expression describing the allowed format of the field's contents. 
	 */
	public FieldPictoConfirm (String sTitle, String sFieldTitle, String sPictoListTitle, 
			                  String sCorrectFormat) {
		super (sTitle, sFieldTitle, sCorrectFormat);
		int iCellDim;	//dimensions for list cells
		
		//Works out the dimensions of all list items according to the current screen's resolution
		iCellDim = (int) (Global.SCREEN_DIM.height / 10);
		iIcoDim = iCellDim - 2 * Global.BUTTON_EDGE;
		iNumItems = (int) (Global.SCREEN_DIM.width / 250);
		
		//Sets up a single-row single-selection item list
		model = new DefaultListModel();
		model.setSize (iNumItems); 
		pictoList = new JList(model);
		pictoList.setSelectionMode (ListSelectionModel.SINGLE_SELECTION);
		pictoList.setLayoutOrientation (JList.HORIZONTAL_WRAP);
		pictoList.setCursor (Cursor.getPredefinedCursor (Cursor.HAND_CURSOR));
		pictoList.setFixedCellWidth (iCellDim);
        pictoList.setFixedCellHeight (iCellDim);
        pictoList.setVisibleRowCount (1);
                
        //Makes each icon is centred within its corresponding cell
        pictoList.setCellRenderer (new DefaultListCellRenderer() {
        	                           @Override
                                       public Component getListCellRendererComponent (JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                                           JLabel label = (JLabel) super.getListCellRendererComponent (list, value, index, isSelected, cellHasFocus);
                                           label.setHorizontalAlignment (JLabel.CENTER);
                                           return label;
                                       }
                                   });
               				
		//Adds the list to the confirmation dialogue.
		changeCentre (Canvas.titledFrame (sPictoListTitle, pictoList, null, true));
					        
		//Sets up the timer for validation and queries: triggered manually (except the first go)
		cronoText = new Timer (Global.FIELD_RESPONSE, this);
		cronoText.setRepeats (false);	
	}
	
	/**
	 * Gets the pictogram's id from the selected pictogram's index.
	 * @param sDefaultPictoId Designated id for the pictogram to fall back on in case 
	 * there is no possible representation for the chosen document name. 
	 * @return String with Id of selected pictogram.
	 */
	public String getPictoId (String sDefaultPictoId) {
		if (ficherosItems != null)
		   return ficherosItems[pictoList.getSelectedIndex()];
		else return sDefaultPictoId;
	}
	
	/**
	 * Resets the dialogue emptying the pictogram list and the text field 
	 * (the latter if required). 
	 * @param bCampos True if fields are to be reset. False if not.
	 */
	public void reset (boolean bCampos) {
		if (bCampos)
		   super.reset();
		ficherosItems = null;
		model.clear();
		refresh();
	}
	
	/**
	 * Overridden method. Tests that the field's text meets format requirements and 
	 * represents a unique document name. If so, a query to search for matching pictograms 
	 * is launched.
	 * @param iIdCampo ID of the field to be tested.
	 */
	protected boolean isCorrect (int iIdCampo) {
		//Text typed in the field
		String sText = getFields()[iIdCampo].trim();
		
		//The field is empty? => empties pictogram list and disables ok button
		if (sText.isEmpty()) {
	       reset (false); 
		   return false;
		} else if (super.isCorrect(iIdCampo)) {
		   	      ImageArray items;		//icon image for each searched pictogram
	    	      String[] foundPictos;		//maps list index of found pictogram to id (after query)
	    	      
	    	      ficherosItems = null;
	    	      
	              //Response waiting time is up? => launch the search and display found pictos
	              if (!cronoText.isRunning()) {
	        
	             	 //Shows busy cursor.
	    		     busy (true);
	        	
	           	     //Displays empty pictogram list
	        	     model.clear(); 
	    		     	                                   
	                 //Strips any number coming after the word for search purposes 
	                 if (sText.trim().matches ("^([a-zA-Z_áéíóú]+[0-9]+)$"))
	                    sText = sText.replaceAll("[0-9]+", "");	
	                        
	                 //Performs the search and filters out pictograms that cannot be displayed
	                 foundPictos = Data.buscaItemsLista (sText.trim());
	                 if ((foundPictos != null) && (foundPictos.length > 0)) { //was the query successful?
	                    if (foundPictos.length >= iNumItems) {
	                       ficherosItems = new String[iNumItems]; 
	                       System.arraycopy (foundPictos, 0, ficherosItems, 0, iNumItems);
	                	} else ficherosItems = foundPictos;
	                    items = new ImageArray (ficherosItems, Global.PICT_FOLDER + Global.SEPARADOR_SYS, 
                                                 Global.PICT_EXT);
	    	            items.resize (iIcoDim, iIcoDim);
	    	     
	    	            //Updates the pictogram list with search results
	                    Icon[] itemsIcons = items.toIcons(); 
	                    for (int i = 0; i < itemsIcons.length; i++)
	    	                model.addElement (itemsIcons[i]);
	                    pictoList.setSelectedIndex (0);	                    
	                  } else pictoList.getSelectionModel().clearSelection();
	                                   
	                  //Resets the field's response timer and the mouse pointer.
	                  cronoText.restart();                  
	                  busy (false);	                                    
	        	  }
	              return true;
	    }
		return false;
	}	
}
